@@ -1,7 +1,7 @@
 """DB-SIG compliant module for communicating with MS SQL servers"""
 
 __author__ = 'Mikhail Denisenko <denisenkom@gmail.com>'
-__version__ = '1.6.12'
+__version__ = '1.7.0'
 
 import logging
 import six
@@ -9,7 +9,7 @@ import os
 from six.moves import xrange
 from . import lcid
 from datetime import date, datetime, time
-from dateutil.tz import tzlocal
+from . import tz
 import socket
 import errno
 import uuid
@@ -25,7 +25,7 @@ from .tds import (
 
 logger = logging.getLogger(__name__)
 
-# comliant with DB SIG 2.0
+# compliant with DB SIG 2.0
 apilevel = '2.0'
 
 # module may be shared, but not connections
@@ -70,7 +70,8 @@ class _Connection(object):
     def autocommit(self, value):
         if self._autocommit != value:
             if value:
-                self._main_cursor._commit(cont=False)
+                if self._conn.tds72_transaction:
+                    self._main_cursor._rollback(cont=False)
             else:
                 self._main_cursor._begin_tran(isolation_level=self._isolation_level)
             self._autocommit = value
@@ -142,21 +143,19 @@ class _Connection(object):
         if not login.port:
             login.port = 1433
         connect_timeout = login.connect_timeout
-        login.query_timeout = login.connect_timeout if login.connect_timeout else login.query_timeout
         err = None
         for host in login.load_balancer.choose():
             try:
                 sock = socket.create_connection(
                     (host, login.port),
                     connect_timeout or 90000)
-                sock.settimeout(login.query_timeout)
                 sock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
             except socket.error as e:
                 err = LoginError("Cannot connect to server '{0}': {1}".format(host, e), e)
                 continue
             try:
                 self._conn = _TdsSocket(self._use_tz)
-                self._conn.login(self._login, sock)
+                self._conn.login(self._login, sock, self._tzinfo_factory)
                 break
             except Exception as e:
                 sock.close()
@@ -167,16 +166,17 @@ class _Connection(object):
             if not err:
                 err = LoginError("Cannot connect to server, load balancer returned empty list")
             raise err
+        sock.settimeout(login.query_timeout)
         self._active_cursor = self._main_cursor = self.cursor()
         if not self._autocommit:
             self._main_cursor._begin_tran(isolation_level=self._isolation_level)
 
-    def __init__(self, server='.', database='', user='', password='', timeout=0,
+    def __init__(self, server='.', database='', user='', password='', timeout=None,
                  login_timeout=60, as_dict=False,
                  appname=None, port=None, tds_version=TDS74,
                  encryption_level=TDS_ENCRYPTION_OFF, autocommit=False,
                  blocksize=4096, use_mars=False, auth=None, readonly=False,
-                 load_balancer=None, use_tz=None):
+                 load_balancer=None, use_tz=None, bytes_to_unicode=True):
         """
         Constructor for creating a connection to the database. Returns a
         Connection object.
@@ -231,7 +231,7 @@ class _Connection(object):
         if use_tz:
             login.client_tz = use_tz
         else:
-            login.client_tz = tzlocal()
+            login.client_tz = tz.local
 
         # that will set:
         # ANSI_DEFAULTS to ON,
@@ -243,17 +243,20 @@ class _Connection(object):
         login.connect_timeout = login_timeout
         login.query_timeout = timeout
         login.server_name = server
-        login.instance_name = instance
+        login.instance_name = instance.upper()  # to make case-insensitive comparison work this should be upper
         login.blocksize = blocksize
         login.auth = auth
         login.readonly = readonly
         login.load_balancer = load_balancer or SimpleLoadBalancer([server])
+        login.bytes_to_unicode = bytes_to_unicode
         self._use_tz = use_tz
         self._autocommit = autocommit
         self._login = login
         self._as_dict = as_dict
         self._isolation_level = 0
         self._dirty = False
+        from .tz import FixedOffsetTimezone
+        self._tzinfo_factory = None if use_tz is None else FixedOffsetTimezone
         self._open()
 
     def __enter__(self):
@@ -282,19 +285,27 @@ class _Connection(object):
         if self.mars_enabled:
             in_tran = self._conn.tds72_transaction
             if in_tran and self._dirty:
-                return _MarsCursor(self, self._conn.create_session())
+                return _MarsCursor(self,
+                                   self._conn.create_session(self._tzinfo_factory),
+                                   self._tzinfo_factory)
             else:
                 try:
-                    return _MarsCursor(self, self._conn.create_session())
+                    return _MarsCursor(self,
+                                       self._conn.create_session(self._tzinfo_factory),
+                                       self._tzinfo_factory)
                 except socket.error as e:
                     if e.errno != errno.ECONNRESET:
                         raise
                 except ClosedConnectionError:
                     pass
                 self._assert_open()
-                return _MarsCursor(self, self._conn.create_session())
+                return _MarsCursor(self,
+                                   self._conn.create_session(self._tzinfo_factory),
+                                   self._tzinfo_factory)
         else:
-            return _Cursor(self, self._conn.main_session)
+            return _Cursor(self,
+                           self._conn.main_session,
+                           self._tzinfo_factory)
 
     def rollback(self):
         """
@@ -360,10 +371,11 @@ class _Cursor(six.Iterator):
     This class represents a database cursor, which is used to issue queries
     and fetch results from a database connection.
     """
-    def __init__(self, conn, session):
+    def __init__(self, conn, session, tzinfo_factory):
         self._conn = conn
         self.arraysize = 1
         self._session = session
+        self._tzinfo_factory = tzinfo_factory
 
     def _assert_open(self):
         if not self._conn:
@@ -389,6 +401,7 @@ class _Cursor(six.Iterator):
         return self
 
     def _callproc(self, procname, parameters):
+        self._ensure_transaction()
         results = list(parameters)
         parameters = self._session._convert_params(parameters)
         self._exec_with_retry(lambda: self._session.submit_rpc(procname, parameters, 0))
@@ -422,6 +435,14 @@ class _Cursor(six.Iterator):
     def spid(self):
         return self._session._spid
 
+    def _get_tzinfo_factory(self):
+        return self._tzinfo_factory
+
+    def _set_tzinfo_factory(self, tzinfo_factory):
+        self._tzinfo_factory = self._session.tzinfo_factory = tzinfo_factory
+
+    tzinfo_factory = property(_get_tzinfo_factory, _set_tzinfo_factory)
+
     def get_proc_return_status(self):
         if self._session is None:
             return None
@@ -432,8 +453,7 @@ class _Cursor(six.Iterator):
     def cancel(self):
         self._assert_open()
         self._conn._try_activate_cursor(self)
-        self._session.send_cancel()
-        self._session.process_cancel()
+        self._session.cancel_if_pending()
 
     def close(self):
         """
@@ -464,28 +484,53 @@ class _Cursor(six.Iterator):
             self._assert_open()
             return fun()
 
+    def _ensure_transaction(self):
+        if not self._conn._autocommit and not self._conn._conn.tds72_transaction:
+            self._conn._main_cursor._begin_tran(isolation_level=self._conn._isolation_level)
+
     def _execute(self, operation, params):
+        self._ensure_transaction()
+        operation = six.text_type(operation)
         if params:
             if isinstance(params, (list, tuple)):
-                names = tuple('@P{0}'.format(n) for n in range(len(params)))
+                names = []
+                pid = 1
+                named_params = {}
+                for val in params:
+                    if val is None:
+                        names.append('NULL')
+                    else:
+                        name = '@P{0}'.format(pid)
+                        names.append(name)
+                        named_params[name] = val
+                        pid += 1
                 if len(names) == 1:
                     operation = operation % names[0]
                 else:
-                    operation = operation % names
-                params = dict(zip(names, params))
+                    operation = operation % tuple(names)
             elif isinstance(params, dict):
                 # prepend names with @
-                rename = dict((name, '@{0}'.format(name)) for name in params.keys())
-                params = dict(('@{0}'.format(name), value) for name, value in params.items())
+                rename = {}
+                named_params = {}
+                for name, value in params.items():
+                    if value is None:
+                        rename[name] = 'NULL'
+                    else:
+                        mssql_name = '@{0}'.format(name)
+                        rename[name] = mssql_name
+                        named_params[mssql_name] = value
                 operation = operation % rename
-            params = self._session._convert_params(params)
-            param_definition = ','.join(
-                '{0} {1}'.format(p.column_name, p.type.get_declaration())
-                for p in params)
-            self._exec_with_retry(lambda: self._session.submit_rpc(
-                SP_EXECUTESQL,
-                [self._session.make_param('', operation), self._session.make_param('', param_definition)] + params,
-                0))
+            if named_params:
+                named_params = self._session._convert_params(named_params)
+                param_definition = u','.join(
+                    u'{0} {1}'.format(p.column_name, p.type.get_declaration())
+                    for p in named_params)
+                self._exec_with_retry(lambda: self._session.submit_rpc(
+                    SP_EXECUTESQL,
+                    [self._session.make_param('', operation), self._session.make_param('', param_definition)] + named_params,
+                    0))
+            else:
+                self._exec_with_retry(lambda: self._session.submit_plain_query(operation))
         else:
             self._exec_with_retry(lambda: self._session.submit_plain_query(operation))
         self._session.find_result_or_done()
@@ -622,8 +667,8 @@ class _Cursor(six.Iterator):
         if not columns:
             self.execute('select top 1 * from [{}] where 1<>1'.format(table_or_view))
             columns = [col[0] for col in self.description]
-        metadata = [Column(name=col, type=self._session.long_string_type(), flags=Column.fNullable) for col in columns]
-        col_defs = ','.join('{} {}'.format(col.column_name, col.type.get_declaration())
+        metadata = [Column(name=col, type=self._conn._conn.NVarChar(4000), flags=Column.fNullable) for col in columns]
+        col_defs = ','.join('{0} {1}'.format(col.column_name, col.type.get_declaration())
                             for col in metadata)
         with_opts = []
         if check_constraints:
@@ -633,17 +678,17 @@ class _Cursor(six.Iterator):
         if keep_nulls:
             with_opts.append('KEEP_NULLS')
         if kb_per_batch:
-            with_opts.append('KILOBYTES_PER_BATCH = {}'.format(kb_per_batch))
+            with_opts.append('KILOBYTES_PER_BATCH = {0}'.format(kb_per_batch))
         if rows_per_batch:
-            with_opts.append('ROWS_PER_BATCH = {}'.format(rows_per_batch))
+            with_opts.append('ROWS_PER_BATCH = {0}'.format(rows_per_batch))
         if order:
-            with_opts.append('ORDER({})'.format(','.join(order)))
+            with_opts.append('ORDER({0})'.format(','.join(order)))
         if tablock:
             with_opts.append('TABLOCK')
         with_part = ''
         if with_opts:
-            with_part = 'WITH ({})'.format(','.join(with_opts))
-        operation = 'INSERT BULK [{}]({}) {}'.format(table_or_view, col_defs, with_part)
+            with_part = 'WITH ({0})'.format(','.join(with_opts))
+        operation = 'INSERT BULK [{0}]({1}) {2}'.format(table_or_view, col_defs, with_part)
         self.execute(operation)
         self._session.submit_bulk(metadata, reader)
         self._session.process_simple_request()
@@ -655,7 +700,7 @@ class _MarsCursor(_Cursor):
             raise Error('Cursor is closed')
         self._conn._assert_open()
         if not self._session.is_connected():
-            self._session = self._conn._conn.create_session()
+            self._session = self._conn._conn.create_session(self._tzinfo_factory)
 
     @property
     def spid(self):
@@ -667,8 +712,7 @@ class _MarsCursor(_Cursor):
 
     def cancel(self):
         self._assert_open()
-        self._session.send_cancel()
-        self._session.process_cancel()
+        self._session.cancel_if_pending()
 
     def close(self):
         """
